#include <stdlib.h>
#include "hyperint.H"
void HyperInt::assign(const ndr_uhyper_int_rep & p0)
    {
	/* this code is kludged from void HyperInt::assign(const
	 * ndr_hyper_int_rep & p0) and will no doubt need to be re-visited */
	HyperInt        High(p0.high);
	HyperInt        Low(p0.low);

	mpz_mul_2exp(&data, &High.data, 32);
	mpz_add(&data, &data, &Low.data);
    }
void HyperInt::assign(const ndr_hyper_int_rep & p0)
{
	HyperInt        High(p0.high);
	HyperInt        Low(p0.low);

	mpz_mul_2exp(&data, &High.data, 32);
	mpz_add(&data, &data, &Low.data);
}

HyperInt::operator ndr_uhyper_int_rep() const
    {
	/* this code is a kludge based on the HyperInt::operator
	 * ndr_hyper_int_rep() const and will no doubt have to be re-visited */
	ndr_uhyper_int_rep x;
	MP_INT          temp;
	mpz_init_set(&temp, &data);
	x.low = mpz_get_ui(&temp);
	mpz_div_2exp(&temp, &temp, 32);
	x.high = mpz_get_si(&temp);
	mpz_clear(&temp);
	if (data.size < 0)
	    {
		x.low = -x.low;
		x.high = (unsigned)-1;
	    }
	return x;
    }
HyperInt::operator ndr_hyper_int_rep() const
{
    /* for negative numbers greater than -4294967296 sign extend into the high
     * word.  this is determined by the size member of data which will be
     * negative if it represents a negative number. */
	ndr_hyper_int_rep x;
	MP_INT          temp;
	mpz_init_set(&temp, &data);
	x.low = mpz_get_ui(&temp);
	mpz_div_2exp(&temp, &temp, 32);
	x.high = mpz_get_si(&temp);
	mpz_clear(&temp);
	if (data.size < 0)
	    {
		x.low = -x.low;
		x.high = -1;
	    }
	return x;
}

HyperInt::HyperInt(const ndr_uhyper_int_rep & p0)
    {
	mpz_init(&data);
	assign(p0);
    }
HyperInt::HyperInt(const ndr_hyper_int_rep & p0)
{
	mpz_init(&data);
	assign(p0);
}

HyperInt & HyperInt::operator = (const ndr_uhyper_int_rep & p0)
    {
	return operator=((const ndr_hyper_int_rep&)p0);
    }
HyperInt & HyperInt::operator = (const ndr_hyper_int_rep & p0)
{
	mpz_clear(&data);
	mpz_init(&data);
	assign(p0);
	return (*this);
}

HyperInt::HyperInt()
{
	mpz_init(&data);
}

HyperInt::HyperInt(const HyperInt & h)
{
	mpz_init(&data);
	mpz_set(&data, &h.data);
}

HyperInt::HyperInt(const unsigned long int &i)
{
	mpz_init(&data);
	mpz_set_ui(&data, i);
}

HyperInt::HyperInt(const signed long int &i)
{
	mpz_init(&data);
	mpz_set_si(&data, i);
}

HyperInt::HyperInt(const char * const p)
{
	mpz_init(&data);
	if (mpz_set_str(&data, p, 0))
		abort();
}

HyperInt::HyperInt(const unsigned char * const p)
{
	mpz_init(&data);
	if (mpz_set_str(&data, (const char *)p, 0))
		abort();
}

HyperInt::~HyperInt()
{
	mpz_clear(&data);
}

HyperInt::operator char *() const
    {
	char * p = new char[mpz_sizeinbase(&data, 10)+ 1];
	char * q = mpz_get_str(p, 10, &data);
	return(p);
    }
HyperInt::operator unsigned char *() const
    {
	unsigned char * p = new unsigned char[mpz_sizeinbase(&data, 10)+ 1];
	unsigned char * q = (unsigned char *)mpz_get_str((char *)p, 10, &data);
	return(p);
    }
ostream & operator << (ostream & o, const HyperInt & h)
{
	size_t          i = mpz_sizeinbase(&h.data, 10);

	/*
	 * work around for memory deallocation problem on hp
	 */

#ifndef __hpux
	char           *p = new char[i + 1];

#else
	char            workbuf[2048];
	char           *p = workbuf;

#endif
	char           *q = mpz_get_str(p, 10, &h.data);

	o << q;

#ifndef __hpux
	delete          p;

#endif

	return (o);
}

ostream & operator << (ostream & o, const HyperInt * const h)
{
	size_t          i = mpz_sizeinbase(&h->data, 10);

	//work around for memory
		//deallocation problem on hp

#ifndef __hpux
			char           *p = new char[i + 1];

#else
			char            workbuf[2048];
	char           *p = workbuf;

#endif
	char           *q = mpz_get_str(p, 10, &h->data);

	o << q;

#ifndef __hpux
	delete          p;

#endif

	return (o);
}
HyperInt::operator unsigned long int () const
{
	return (mpz_get_ui(&data));
}
HyperInt::operator signed long int () const
{
	return (mpz_get_si(&data));
}

HyperInt        operator + (const HyperInt & p0, const HyperInt & p1)
{
	return HyperInt(p0, p1, (HyperIntPlusOperator *) 0);
}

HyperInt::HyperInt(const HyperInt & p0, const HyperInt & p1,
	 HyperIntPlusOperator *)
{
	mpz_init(&data);
	mpz_add(&data, &p0.data, &p1.data);
}
HyperInt        operator - (const HyperInt & p0, const HyperInt & p1)
{
	return HyperInt(p0, p1, (HyperIntMinusOperator *) 0);
}

HyperInt::HyperInt(const HyperInt & p0, const HyperInt & p1,
	 HyperIntMinusOperator *)
{
	mpz_init(&data);
	mpz_sub(&data, &p0.data, &p1.data);
}
HyperInt operator *(const HyperInt & p0, const HyperInt & p1)
{
	return HyperInt(p0, p1, (HyperIntMultiplicationOperator *) 0);
}

HyperInt::HyperInt(const HyperInt & p0, const HyperInt & p1,
	 HyperIntMultiplicationOperator *)
{
	mpz_init(&data);
	mpz_mul(&data, &p0.data, &p1.data);
}
HyperInt        operator / (const HyperInt & p0, const HyperInt & p1)
{
	return HyperInt(p0, p1, (HyperIntDivisionOperator *) 0);
}

HyperInt::HyperInt(const HyperInt & p0, const HyperInt & p1,
	 HyperIntDivisionOperator *)
{
	mpz_init(&data);
	mpz_div(&data, &p0.data, &p1.data);
}
HyperInt        operator % (const HyperInt & p0, const HyperInt & p1)
{
	return HyperInt(p0, p1, (HyperIntModuloOperator *) 0);
}

HyperInt::HyperInt(const HyperInt & p0, const HyperInt & p1,
	 HyperIntModuloOperator *)
{
	mpz_init(&data);
	mpz_mod(&data, &p0.data, &p1.data);
}

HyperInt & HyperInt::operator = (const HyperInt & p0)
{
	mpz_clear(&data);
	mpz_init(&data);
	mpz_set(&data, &p0.data);
	return (*this);
}

HyperInt & HyperInt::operator = (const unsigned long int &p0)
{
	mpz_set_ui(&data, p0);
	return (*this);
}

HyperInt & HyperInt::operator = (const signed long int &p0)
{
	mpz_set_si(&data, p0);
	return (*this);
}

HyperInt & HyperInt::operator *= (const HyperInt & p0)
{
	mpz_mul(&data, &data, &p0.data);
	return (*this);
}

HyperInt & HyperInt::operator /= (const HyperInt & p0)
{
	mpz_div(&data, &data, &p0.data);
	return (*this);
}

HyperInt & HyperInt::operator %= (const HyperInt & p0)
{
	mpz_mod(&data, &data, &p0.data);
	return (*this);
}

HyperInt & HyperInt::operator += (const HyperInt & p0)
{
	mpz_add(&data, &data, &p0.data);
	return (*this);
}

HyperInt & HyperInt::operator -= (const HyperInt & p0)
{
	mpz_sub(&data, &data, &p0.data);
	return (*this);
}
int             HyperInt::operator == (const HyperInt & p0) const
{
	int result = mpz_cmp(&data, &p0.data);

	if (!result)
		return (1 == 1);
	else
		return (1 == 0);
}
int             HyperInt::operator != (const HyperInt & p0) const
{
	int             result = mpz_cmp(&data, &p0.data);

	if (result)
		return (1 == 1);
	else
		return (1 == 0);
}
int             HyperInt::operator < (const HyperInt & p0) const
{
	int             result = mpz_cmp(&data, &p0.data);

	if (result < 0)
		return (1 == 1);
	else
		return (1 == 0);
}
int             HyperInt::operator <= (const HyperInt & p0) const
{
	int             result = mpz_cmp(&data, &p0.data);

	if (result <= 0)
		return (1 == 1);
	else
		return (1 == 0);
}
int             HyperInt::operator > (const HyperInt & p0) const
{
	int             result = mpz_cmp(&data, &p0.data);

	if (result > 0)
		return (1 == 1);
	else
		return (1 == 0);
}
int             HyperInt::operator >= (const HyperInt & p0) const
{
	int             result = mpz_cmp(&data, &p0.data);

	if (result >= 0)
		return (1 == 1);
	else
		return (1 == 0);
}
