#ifndef hyperintDEFINED
#define hyperintDEFINED
/*
    HyperInt class for manipulating arbitrary precision signed integer values
*/
#include <iostream.h>
#include <dce/idlbase.h>
#include "hyperint-phonytypes.H"
extern "C" 
   {
#include <gmp.h>
   }
   
class HyperInt
    {
	public:
	    HyperInt();
	    HyperInt(const HyperInt &);
	    HyperInt(const ndr_hyper_int_rep &);
	    HyperInt(const ndr_uhyper_int_rep &);
	    HyperInt(const unsigned long int &);
	    HyperInt(const signed long int &);
	    HyperInt(const char * const);
	    HyperInt(const unsigned char * const);
	    ~HyperInt();
	    friend ostream & operator<<(ostream & , const HyperInt &);
	    friend ostream & operator<<(ostream & , const HyperInt * const);
	    operator unsigned long int() const;
	    operator signed long int() const;
	    operator ndr_hyper_int_rep() const;
	    operator ndr_uhyper_int_rep() const;
	    operator char*() const;
	    operator unsigned char*() const;
	    friend HyperInt operator+(const HyperInt &, const HyperInt & ) ;
	    friend HyperInt operator-(const HyperInt &, const HyperInt & ) ;
	    friend HyperInt operator*(const HyperInt &, const HyperInt & ) ;
	    friend HyperInt operator/(const HyperInt &, const HyperInt & ) ;
	    friend HyperInt operator%(const HyperInt &, const HyperInt & ) ;
	    HyperInt & operator=(const HyperInt &);
	    HyperInt & operator=(const unsigned long int &);
	    HyperInt & operator=(const signed long int &);
	    HyperInt & operator=(const ndr_hyper_int_rep &);
	    HyperInt & operator=(const ndr_uhyper_int_rep &);
	    HyperInt & operator*=(const HyperInt &);
	    HyperInt & operator/=(const HyperInt &);
	    HyperInt & operator%=(const HyperInt &);
	    HyperInt & operator+=(const HyperInt &);
	    HyperInt & operator-=(const HyperInt &);
	    int operator==(const HyperInt & ) const;
	    int operator!=(const HyperInt & ) const;
	    int operator<(const HyperInt & ) const;
	    int operator<=(const HyperInt & ) const;
	    int operator>(const HyperInt & ) const;
	    int operator>=(const HyperInt & ) const;
	private:
	    MP_INT data;
	    HyperInt(const HyperInt &, const HyperInt &, 
	    		HyperIntPlusOperator *);
	    HyperInt(const HyperInt &, const HyperInt &, 
	    		HyperIntMinusOperator *);
	    HyperInt(const HyperInt &, const HyperInt &, 
	    		HyperIntDivisionOperator *);
	    HyperInt(const HyperInt &, const HyperInt &, 
	    		HyperIntMultiplicationOperator *);
	    HyperInt(const HyperInt &, const HyperInt &, 
	    		HyperIntModuloOperator *);
	    void assign(const ndr_hyper_int_rep & p0);
	    void assign(const ndr_uhyper_int_rep & p0);
    };
#endif
