/*
    test-cvt-1.C
*Test HyperInt conversions between ndr and mint.
*/
#include <string.h>
#include <stdlib.h>
#include <iostream.h>
#include <memory.h>
#include <Args.h>
#include <Stopwatch.h>
#include <hyperint.H>
void suite2()
    {
	
	MP_INT one;
	mpz_init_set_ui(&one, 1);
	MP_INT temp;
	mpz_init_set_si(&temp, 1);
	int i;
	for (i = 0; i < 64; i++)
	    {
		char strbuf[256];
		mpz_mul_2exp(&temp, &one, i);
		mpz_get_str(strbuf, 10, &temp);
		cout << i << "\t" << strbuf << endl;
	    }
	mpz_clear(&one);
	mpz_clear(&temp);
    }
void suite0()
    {
	MP_INT one;
	mpz_init_set_ui(&one, 1);
	MP_INT minus_one;
	mpz_init_set_si(&minus_one, -1);
	MP_INT zero;
	long init = 20;
	mpz_init_set_si(&zero, init);

	int lc;
	for (lc = 0; lc < init * 2; lc++)
	    {
		char strbuf[256];
		mpz_get_str(strbuf, 16, &zero);
		cout << "String " << strbuf << endl << endl;
		mpz_sub_ui(&zero, &zero, 1);
	    }

	mpz_clear(&zero);
	mpz_clear(&minus_one);
	mpz_clear(&one);
    }
void suite1()
    {
	unsigned long N = 0;
	Stopwatch timer;
	const unsigned long period = 10000;
	unsigned long iteration_count = period;
	const HyperInt a((const signed long)1);
	HyperInt b((const unsigned long)10);
	HyperInt c((const signed long)0);
	ndr_hyper_int_rep x;
	ndr_hyper_int_rep y;
	memset(&x, 0, sizeof(x));
	memset(&y, 0, sizeof(y));
	int LoopControl = 1;
	cout << "Initial values" << endl << "period = " << period << endl << 
	"b == " << b << endl << "c == " << c << endl << "x== " << x << endl << 
	"y == " << y << endl << endl;
	timer.stop();
	timer.reset();
	timer.start();

	while (LoopControl)
	    {
		N += 1;
		b -= a;
		x = b;
		c = x;
		y = c;
		if (++iteration_count >= period)
		    {
			iteration_count = 0;
			cout << "Iteration Count " << N << endl
			<< "b == " << b << endl 
			<< "c == " << c << endl 
			<< "x == " << x << endl 
			<< "y == " << y << endl 
			<< "user == " << timer.user() << endl
			<< "system == " << timer.system() << endl
			<< "real == " << timer.real() << endl;
		    }
		if (b != c || memcmp(&x, &y, sizeof(ndr_hyper_int_rep)))
		    {
			cerr << "Conversion failure" << endl;
			cerr << "b == " << b << endl;
			cerr << "c == " << c << endl;
			cerr << "x == " << x << endl;
			cerr << "y == " << y << endl;
			LoopControl = 0;
		    }
	    }
    }
void suite3()
    {
	char * input_str = "12345678901234567890";
	cout << "Input string is " << input_str << endl;
	HyperInt a(input_str);
	cout << "HyperInt value is " << a << endl;
	char * output_str = (char*)a;
	cout << "Output string is " << output_str << endl;
	if (strcmp(input_str, output_str))
	    cerr << "As you can see they are not identical!" << endl;
	delete output_str;
    }
void suite4()
    {
	unsigned char * input_str = (unsigned char *)"12345678901234567890";
	cout << "Input string is " << (char *)input_str << endl;
	HyperInt a(input_str);
	cout << "HyperInt value is " << a << endl;
	unsigned char * output_str = (unsigned char*)a;
	cout << "Output string is " << (char *)output_str << endl;
	if (strcmp((char *)input_str, (char *)output_str))
	    cerr << "As you can see they are not identical!" << endl;
	delete output_str;
    }
main(int argc,  char * argv[])
    {
	Args arg(argc, argv, "");
	int i;
	if (!arg.nargs())
	    cout << "Please specify one or more test suites" << endl;
	for (i = 0; i < arg.nargs(); i++)
	    {
		if (arg.arg(i) != 0)
		    {
			switch (atoi(arg.arg(i)))
			    {
				case 0:
				    cout << "Executing case 0" << endl;
				    suite0();
				    break;
				case 1:
				    cout << "Executing case 1" << endl;
				    suite1();
				    break;
				case 2:
				    cout << "Executing case 2" << endl;
				    suite2();
				    break;
				case 3:
				    cout << "Executing case 3" << endl;
				    suite3();
				    break;
				case 4:
				    cout << "Executing case 4" << endl;
				    suite4();
				    break;
				default:
				    cerr << "Invalid test suite number " <<
				    arg.arg(i) << endl;
			    }
		    }
	    }
    }
